@@ -1,29 +1,34 @@
+import glob
 from random import choice, randint,randrange
 from datetime import datetime
 import asyncio
 import os
 from random import choice, random
+import shutil
 from time import sleep, time
 from telethon.sync import TelegramClient
 from telethon.sync import events, functions, errors
 import telethon
-from telethon.tl.functions.contacts import GetContactsRequest, ImportContactsRequest, DeleteContactsRequest, ResolveUsernameRequest
-
-from telethon.tl.types import InputPhoneContact, InputGeoPoint, InputPeerChannel, InputChannel, InputChatPhoto, InputPhoto, InputChatUploadedPhoto, InputFile, InputPeerUser
+from telethon.tl.functions.contacts import GetContactsRequest, ImportContactsRequest, DeleteContactsRequest, ResolveUsernameRequest,UnblockRequest,BlockRequest
+from telethon.tl.functions.account import UpdateStatusRequest,SetPrivacyRequest
+# from telethon.tl.functions.stats import 
+from telethon.tl.types import InputPhoneContact, InputGeoPoint, InputPeerChannel, InputChannel, InputChatPhoto, InputPhoto, InputChatUploadedPhoto, InputFile, InputPeerUser,MessageEntityTextUrl
 
+from telethon.tl.types import MessageEntityCode
 # ======== token
 from telethon.tl.functions.channels import GetParticipantsRequest
 from telethon.tl.functions.help import GetUserInfoRequest
-from telethon.tl.types import ChannelParticipant
+from telethon.tl.types import ReactionEmoji
 from telethon.tl.types import InputChannel, ChannelAdminLogEventsFilter, ChannelParticipantsSearch
 from telethon.tl.functions.account import UpdateProfileRequest
 
 from telethon.tl.functions.channels import JoinChannelRequest, LeaveChannelRequest, CreateChannelRequest, CheckUsernameRequest, UpdateUsernameRequest, DeleteChannelRequest, EditPhotoRequest
 from telethon.tl.functions.photos import UploadProfilePhotoRequest
-from telethon.tl.functions.messages import GetMessagesViewsRequest,GetMessagesReactionsRequest
-from telethon.tl.functions.users import GetFullUserRequest
+from telethon.tl.functions.messages import GetMessagesViewsRequest,GetMessagesReactionsRequest,SendReactionRequest,SendVoteRequest
+from telethon.tl.functions.users import GetUsersRequest,GetFullUserRequest
 from telethon.tl.types import InputUserSelf
 from telethon.tl.functions.messages import ImportChatInviteRequest
+from telethon import functions, types
 # from telethon.tl.functions.channels import 
 # CheckUsernameRequest()
 import os,json
@@ -34,9 +39,12 @@ from .utils import *
 from starco.proxy import PROXY
 import nest_asyncio
 nest_asyncio.apply()
+from starco.utils import translator
+
+
 
 class TlgApp:
-    def __init__(self, number, key='17203958', hash='82cefc4001e057c9d1488ab90e23d54f', loop=None, debug_mode=True,country_diring=False,auto_connect=True, **kwargs) -> None:
+    def __init__(self, number, key='17203958', hash='82cefc4001e057c9d1488ab90e23d54f', loop=None, debug_mode=True,country_diring=False,reset_json=False,auto_connect=True,disable_proxy=False,just_data=False, **kwargs) -> None:
         '''
         relative_path='.'
         path_list =['accounts',number]
@@ -54,23 +62,32 @@ class TlgApp:
         
         path_list:list = kwargs.get('path_list', ['accounts',str(self.number_int)])
         js_path_list= kwargs.get('js_path_list', ['accounts',str(self.number_int)])
-        
+        self.disable_proxy = disable_proxy
         if country_diring:
             path_list.insert(-1,get_country_name(self.number))
             js_path_list.insert(-1,get_country_name(self.number))
 
         self.base_path = path_maker(path_list, relative_path)
-        self.json_path = path_maker(js_path_list, relative_path)
-
+        self.json_path = path_maker(js_path_list, relative_path)+f"/{self.number}.json"
+        if just_data:return
+        if reset_json:
+            try:os.remove(self.json_path)
+            except:pass        
         self.key = key
         self.hash = hash
 
         self.path = f'{self.base_path}/{session_number(self.number_int)}'
-
         self.status = False
         if type(loop) == type(None):
-            self.loop = asyncio.new_event_loop()
-            asyncio.set_event_loop(self.loop)
+            asyncio.get_event_loop().close()
+            if asyncio.get_event_loop().is_closed():
+                self.loop = asyncio.new_event_loop()
+                asyncio.set_event_loop(self.loop)
+                for i in glob.glob(f"{self.base_path}/*"):
+                    if i in [self.json_path , self.path]:continue
+                    try:os.remove(i)
+                    except:pass
+
         self.client: TelegramClient = None
         self.init_client()
         self.js('last_check_time',int(time()))
@@ -86,19 +103,24 @@ class TlgApp:
     async def _get_status(self):
         try:
             res = await self.client.get_me()
+            try:photo = res.photo.photo_id
+            except:photo =None
+
+            self.js('photo', photo)
+            self.js('first_name', res.first_name)
+            self.js('last_name', res.last_name)
+            # self.js('bio', res.bio)
+            self.js('username', res.username)
         except:
             return False
         if res == None:
             return False
         return True
-
-    def get_status(self):
-
-        return self.loop.run_until_complete(self._get_status())
-
+   
     def connect(self):
         async def act(self: TlgApp):
             try:
+                # if await self.client.is_user_authorized():
                 await self.client.connect()
                 return True
             except:pass
@@ -107,62 +129,78 @@ class TlgApp:
         self.loop.run_until_complete(act(self))
 
     def init_client(self):
-        versions = ['9.0', "8.9", "8.4", "9.2", "9.1", "9.3", "8.6", "7.9"]
-        appvs = ["7.84", "7.43", "8.0", "4.9", "7.19","7.21", "7.34", "7.42", "7.86", "8.0", "1.34"]
-        langs = ["en"]
-        device = ''
-        # choice(devices)
-        version = choice(versions)
-        appv = choice(appvs)
-        lang = choice(langs)
-            
-        extra = {
-            'device_model': 'Telegram Desktop 4.6.5',
-        }
-        prx = self.js('proxy')
-        if prx:
-            extra['proxy'] = PROXY(**prx).make_proxy()
+        api_id = self.js('app_id')
+        app_hash = self.js('app_hash')
+        proxy = self.js('proxy')
+        
+        if self.proxy==None and proxy and proxy!='null':
+            self.proxy=proxy
 
-        elif self.proxy:
+        if self.js('app_id'):
+            self.key =api_id
+            self.hash=app_hash
+
+        system_versions = ['5.15.1','5.11.3','4.15.5','5.10.4','3.11.8','4.1.1']
+        system_version =  choice(system_versions) if not self.js('system_version') else self.js('system_version')
+        device_models =[f"Telegram Desktop {i}" for i in ['4.12.2','4.11.3','4.11.5','4.10.3','4.11.8','4.12.1']]
+        device_model =  choice(device_models) if not self.js('device_model') else self.js('device_model')
+        app_version =  '1.6.7' if not self.js('app_version') else self.js('app_version')
+        extra = {
+            'app_version': app_version,
+            'device_model': device_model,
+            'system_version': system_version,
+            
+        }
+        if self.proxy and not self.disable_proxy:
+            self.js('proxy',self.proxy)
             extra['proxy'] = PROXY(**self.proxy).make_proxy()
 
         self.client = TelegramClient(
             session=self.path, 
             api_id=self.key,
-              api_hash=self.hash,
-              timeout=self.timeout, 
+            api_hash=self.hash,
+            timeout=self.timeout,
               **extra)
         self.status = True
         def setjs(self,k,v):
-            if self.js(k):
+            if not self.js(k):
                 self.js(k,v)
-        setjs(self,'session_file',self.number)
-        setjs(self,'phone',self.number)
-        setjs(self,'register_time',int(time()))
-        setjs(self,'app_id',self.key)
-        setjs(self,'app_hash',self.hash)
-        setjs(self,'app_version',version)
-        setjs(self,'avatar',"null")
-        setjs(self,'device',device)
-        setjs(self,'sex',"null")
-        setjs(self,'system_lang_pack',lang)
-        setjs(self,'ipv6',self.client._use_ipv6)
+        if self.js('session_file')==None:
+            setjs(self,'session_file',self.number)
+            setjs(self,'phone',self.number)
+            setjs(self,'register_time',int(time()))
+            setjs(self,'app_id',self.key)
+            setjs(self,'app_hash',self.hash)
+            setjs(self,'sdk','sdk 23')
+            setjs(self,'app_version',app_version)
+            setjs(self,'device',device_model)
+            setjs(self,'last_check_time',int(time()))
+            setjs(self,'avatar',"null")
+            setjs(self,'first_name',"null")
+            setjs(self,'last_name',"null")
+            setjs(self,'username',"null")
+            setjs(self,'sex',"null")
+            setjs(self,'lang_pack',"en")
+            setjs(self,'system_lang_pack',"en")
+            setjs(self,'ipv6',False)
+            setjs(self,'password_2fa', '')
         setjs(self,'proxy', self.proxy)
         
+        
         if self.auto_connect:            
             self.connect()
 
-    def js(self,key,value=None):
+    def js(self,key,value=None,do_none=False):
         if self.json_path:
             # print(key,value)
-            js_path = f"{self.json_path}/{self.number}.json"
+            js_path = f"{self.json_path}"
             
             try:
                 with open(js_path,'r') as f:
                     data = f.read()
                 data =json.loads(data)
             except:data={}
-            if type(value)==type(None):
+            if type(value)==type(None) and not do_none:
                 return data.get(key)
             data[key]=value
             with open(js_path,'w') as f:
@@ -220,10 +258,6 @@ class TlgApp:
             out = self.loop.run_until_complete(self.sign_in(code, hash))
             if out[0]:
                 self.d['step'] = 0
-                # me = self.client.get_me()
-                # self.js('first_name', me.first_name)
-                # self.js('last_name', me.last_name)
-                # self.js('username', me.username)
             if out[1] == 'need_pass2':
                 self.d['step'] = 3
                 self.d['code'] = code
@@ -238,10 +272,6 @@ class TlgApp:
                 self.sign_in(code, hash, p2fa))
             if out[0]:
                 self.d['step'] = 0
-                # me = self.client.get_me()
-                # self.js('first_name', me.first_name)
-                # self.js('last_name', me.last_name)
-                # self.js('username', me.username)
                 self.js('password_2fa',p2fa)
             return out[1]
         self.d['step'] = 0
@@ -282,11 +312,12 @@ class TlgApp:
             if have error session been removed
         '''
         try:
-
-            auth = await self.client.send_code_request(self.number)
-            auth_hahs = auth.phone_code_hash
-
-            return True, auth_hahs
+            if not await self.client.is_user_authorized():
+                auth = await self.client.send_code_request(self.number)
+                auth_hahs = auth.phone_code_hash
+                return True, auth_hahs
+            else:
+                return False, '⚠️خطا در ارسال کد !'
         except errors.PhoneNumberBannedError:
             return False, f'⚠️خطا در ارسال کد به شماره {self.number} شماره شما از تلگرام مسدود شده است !'
         except errors.PhoneNumberInvalidError:
@@ -306,16 +337,17 @@ class TlgApp:
 
     async def sign_in(self, code, hash, p2fa=None):
         try:
-            await self.client.sign_in(phone=f'{self.number}', code=(code), phone_code_hash=hash)
-            return True, ''
-
+            if not await self.client.is_user_authorized():
+                await self.client.sign_in(phone=f'{self.number}', code=(code), phone_code_hash=hash)
+                return True, ''
+            else:
+                return False, 'unknow_error'
         except errors.SessionPasswordNeededError as e:
             if p2fa:
                 try:
                     await self.client.sign_in(password=str(p2fa))
                     return True, ''
                 except Exception as e:
-
                     return False, "wrong_pass2"
             else:
                 return False, "need_pass2"
@@ -357,18 +389,38 @@ class TlgApp:
                 async for dialog in self.client.iter_dialogs():
                     tmp = dialog.to_dict()
                     if tmp.get('_') == 'Dialog' and tmp.get('name') == 'Telegram':
-                        code = dialog.message.message
-                        if 'Login code' in code:
-                            find = re.findall('Login code: (\d{5}).',code)
-                            if find:
-                                return find[0]
-                    break
-                        
-
+                        code = translator(dialog.message.message,'en')
+                        # if 'Login code' in code:
+                        find = re.findall('(\d{5}).',code)
+                        if find:
+                            return find[0],dialog.message.date
+                    # break
             except Exception as e:
                 self.debug.debug(e)
         return self.loop.run_until_complete(act(self))
 
+    def active_sessions(self):
+        async def act(self: TlgApp):
+            out=[]
+            async with self.client as client:
+                result = await client(functions.account.GetAuthorizationsRequest())
+                for i in result.authorizations:
+                    i = i.to_dict()
+                    out+=[i]
+            return out
+
+        return self.loop.run_until_complete(act(self))
+
+    def destroy_session(self,session_hash:int):
+        async def act():
+            try:
+                await self.client(functions.account.ResetAuthorizationRequest(hash=int(session_hash)))
+                return True
+            except Exception as e:self.debug.debug(e)
+            return False
+
+        return self.loop.run_until_complete(act())
+
     def destroy_others_session(self):
         async def act(self: TlgApp):
             async with self.client as client:
@@ -376,13 +428,14 @@ class TlgApp:
                 for i in result.authorizations:
                     i = i.to_dict()
                     if not i['current']:
-                        await client(functions.account.ResetAuthorizationRequest(hash=i['hash']))
+                        try:await client(functions.account.ResetAuthorizationRequest(hash=i['hash']))
+                        except:pass
 
         return self.loop.run_until_complete(act(self))
 
     def remove_session(self):
         self.disconnect()
-        try:os.remove(self.path)
+        try:shutil.rmtree(self.base_path)
         except:pass
 
     def just_this_sessions_is_active(self):
@@ -393,15 +446,17 @@ class TlgApp:
                 current_exist = False
                 for i in result.authorizations:
                     i = i.to_dict()
-                    print(i)
                     if i['current']:
                         current_exist = True
                 res = len(result.authorizations) == 1 and current_exist
             return res
         return self.loop.run_until_complete(act(self))
 
-    def do_change_or_set_2fa(self, old_p2fa: str, new_p2fa: str):
-        async def act(self: TlgApp, old_p2fa, new_p2fa):
+    def do_change_or_set_2fa(self, new_p2fa: str,old_p2f=None):
+        async def act():
+            if not old_p2f:
+                old_p2fa = self.js('password_2fa')
+            else:old_p2fa=old_p2f
             if old_p2fa == '':
                 try:
                     await self.client.connect()
@@ -423,7 +478,7 @@ class TlgApp:
                     return False
             return True
 
-        return self.loop.run_until_complete(act(self, old_p2fa, new_p2fa))
+        return self.loop.run_until_complete(act())
 
     def change_info(self,  first_name=None, last_name=None, bio=None):
         async def act():
@@ -440,7 +495,6 @@ class TlgApp:
         async def act():
             try:
                 pic = await self.client.upload_file(photo_path)
-                print(pic)
                 await self.client(UploadProfilePhotoRequest(file=pic))
                 return True
             except Exception as e:
@@ -450,21 +504,38 @@ class TlgApp:
 
         return self.loop.run_until_complete(act())
 
+    def send_message(self,entity, message,delete_dialog=False,json_formatting_entities=None,**kargs):
+        '''
+        file:str=file_path
+        '''
+        async def act():
+            try:
+                file= kargs.get('file')
+                if file:
+                    kargs.pop('file')
+                    file=await self.client.upload_file(file)
+                if json_formatting_entities:
+                    out = []
+                    for i in json.loads(json_formatting_entities):
+                        if i['type']=='text_link':
+                            out+=[MessageEntityTextUrl(i['offset'],i['length'],i['url'])]
+                        elif i['type']=='code':
+                            out+=[MessageEntityCode(i['offset'],i['length'])]
+                        
+                    kargs['formatting_entities'] = out
 
-    def send_message(self,entity, message,delete_dialog=False,**kargs):
-        async def act(self:TlgApp,entity, message,**kargs):
-            res = await self.client.send_message(entity, message,**kargs)
-            if delete_dialog:
-                await self.client.delete_dialog(entity)
-            return res
-        return self.loop.run_until_complete(act(self,entity, message,**kargs))
+                res = await self.client.send_message(entity, message,file=file,**kargs,)
+                if delete_dialog:
+                    await self.client.delete_dialog(entity)
+                return res
+            except Exception as e:self.debug.debug(e)
+        return self.loop.run_until_complete(act())
         
     def download(self,entity,size,out_file_name):
         async def act(self:TlgApp,entity,size,out_file_name):
             async for message in self.client.iter_messages(entity,limit=1, reverse=False):
                 try:                
                     if type(message.media)!=type(None):
-                        print(message.media)
                         gsize=message.media.document.size
                         if size==gsize:
                             await self.client.download_media(message,out_file_name)
@@ -503,8 +574,8 @@ class TlgApp:
             
         return self.loop.run_until_complete(_seen_post())
         
-    def react_post(self, entity, ids:list[int]=[],limit=5):
-
+    def react_post(self, entity,reac:list, ids:list[int]=[],limit=5):
+        if not isinstance(reac,list):reac=[reac]
         async def _react_post():
             '''
             if ids==[] use limit
@@ -515,7 +586,11 @@ class TlgApp:
                     async for msg in self.client.iter_messages(entity, limit=limit):
                         list_of_id += [int(msg.id)]
                 else:list_of_id = ids
-                await self.client(GetMessagesReactionsRequest(peer=entity, id=list_of_id))
+                reaction=[ReactionEmoji(
+                    emoticon=choice(reac)
+                )]
+                for msg_id in ids:
+                    await self.client(SendReactionRequest(peer=entity,msg_id=msg_id,reaction=reaction))
 
                 return True
             except Exception as e:
@@ -523,12 +598,22 @@ class TlgApp:
             return False
             
         return self.loop.run_until_complete(_react_post())
-
+    
+    def vote(self, entity, msg_id:int,option:str):
+        async def act():
+            try:
+                await self.client(SendVoteRequest(peer=entity,msg_id=msg_id,options=[option.encode()]))
+                return True
+            except Exception as e:
+                self.debug.debug(e)
+            return False
+            
+        return self.loop.run_until_complete(act())
+    
     def join_channel(self, channel):
         async def _join_channel():
             try:
                 res = await self.client(JoinChannelRequest(channel))
-                print(res)
                 return True
             except Exception as e:
                 self.debug.debug(e)
@@ -720,10 +805,26 @@ class TlgApp:
 
         self.client.run_until_disconnected()
 
-    def get_user_info(self, PeerUser):
+    def get_user_info(self, per_user,to_dict=False):
+        '''
+        send 1 username
+        '''
         async def _get_user_info():
-            res = await self.client(GetFullUserRequest(PeerUser))
-            return res
+            userls = per_user
+            if not isinstance(userls,list):userls=[userls]
+            try:
+                res = await self.client(GetUsersRequest(userls))
+                if to_dict:
+                    out=[]
+                    for i in res:
+                        out+=[i.to_dict()]
+                    out=out[-1]
+                    return out
+                res=res[-1]
+                return res
+            except Exception as e:
+                print(e)
+            return {}
         return self.loop.run_until_complete(_get_user_info())
 
     def delete_old_dialoges(self,after_sec=864000):
@@ -734,6 +835,7 @@ class TlgApp:
                 async for dialog in self.client.iter_dialogs():
                     tmp = dialog.to_dict()
                     if tmp.get('_') == 'Dialog':
+                        if tmp['entity'].to_dict().get('_')!='User':continue
                         send_time = tmp['message'].to_dict()['date'].timestamp()
                         past = int((int(time()) - send_time))
                         if past > after_sec:
@@ -770,7 +872,7 @@ class TlgApp:
             return out
         return self.loop.run_until_complete(_cp_posts())
         
-    def get_members_info(self,username=None,id=None,access_hash=None):
+    def get_members_info(self,username=None,gid=None,hash=None):
         '''
         username for public group
         id,access hash for private
@@ -782,6 +884,11 @@ class TlgApp:
                 o= await self.client(ResolveUsernameRequest(username))
                 id = o.chats[0].id
                 access_hash = o.chats[0].access_hash
+            else:
+                id=gid
+                access_hash=hash
+            id = int(str(id))
+            access_hash = int(str(access_hash))
             chat = InputChannel(id,access_hash)
             async for i in self.client.iter_participants( chat, limit=10000, aggressive=True):
                     try:
@@ -791,6 +898,7 @@ class TlgApp:
         return self.loop.run_until_complete(_get_group_members_info())
 
     def join_private_group(self, group_hash_id):
+
         '''return id,access_hash'''
         async def _join_group():
             try:
@@ -804,4 +912,104 @@ class TlgApp:
             except Exception as e:
                 print(e)
             return None,None
-        return self.loop.run_until_complete(_join_group())
\ No newline at end of file
+        return self.loop.run_until_complete(_join_group())
+    
+    def ative_last_seen(self):
+        '''privacy'''
+        async def act():
+            try:
+                await self.client(SetPrivacyRequest(types.InputPrivacyKeyStatusTimestamp(),[types.InputPrivacyValueDisallowAll()]))
+                return True
+            except Exception as e:
+                print(e)
+            return False
+        return self.loop.run_until_complete(act())
+    
+    def remove_proxy(self):
+        self.js('proxy',None,True)
+        return True
+    
+    def disable_show_phone_number(self):
+        '''privacy'''
+        async def act():
+            try:
+                await self.client(
+                    SetPrivacyRequest(
+                        types.InputPrivacyKeyPhoneNumber(),
+                        [types.InputPrivacyValueDisallowAll()]
+                        ))
+                return True
+            except Exception as e:
+                print(e)
+            return False
+        return self.loop.run_until_complete(act())
+    
+    def get_last_message(self,entity):
+        async def act():
+            try:
+                out = await self.client.get_messages(entity)
+                return out[0].message
+            except Exception as e:
+                self.debug.debug(e)
+        return self.loop.run_until_complete(act())
+    
+    def get_me(self):
+        async def act():
+            try:
+                out = await self.client.get_me()
+                return out
+            except Exception as e:
+                self.debug.debug(e)
+        return self.loop.run_until_complete(act())
+
+    def get_account_status(self):
+        '''
+        return 'limit' , limit string time
+        return 'ok' , ''
+        return 'ban,''
+        '''
+        ban_pm='Unfortunately, some phone numbers may trigger a harsh response from our anti-spam systems'
+        self.send_message('SpamBot','/start')
+        sleep(2)
+        last_message = self.get_last_message('SpamBot')
+        print(last_message)
+        if 'your account is now limited' in last_message:
+            pattern = "your account is now limited until (.*?UTC)"
+            stime = re.findall(pattern,last_message)[0]
+            return 'limit' , stime
+        elif 'Unfortunately, some actions can trigger a harsh' in last_message:
+            return 'suspand' ,''
+        elif 'Good news, no limits are currently applied to your account' in last_message:
+            return 'ok',''
+        elif ban_pm in last_message:
+            return 'ban',''
+                
+
+    def do_online(self):
+        async def act(self:TlgApp):
+            res = await self.client(UpdateStatusRequest(False))
+            return res
+        return self.loop.run_until_complete(act(self))
+    
+    def st_online_status(self,online=True):
+        async def act(self:TlgApp):
+            res = await self.client(UpdateStatusRequest(not online))
+            return res
+        return self.loop.run_until_complete(act(self))
+    def log_out(self):
+        async def act(self:TlgApp):
+            res = await self.client.log_out()
+            return res
+        return self.loop.run_until_complete(act(self))
+
+    def unblock(self,uid:str):
+        async def act(self:TlgApp):
+            res = await self.client(UnblockRequest(uid))
+            return res
+        return self.loop.run_until_complete(act(self))
+
+    def block(self,uid:str):
+        async def act(self:TlgApp):
+            res = await self.client(BlockRequest(uid))
+            return res
+        return self.loop.run_until_complete(act(self))
\ No newline at end of file
